/* File level history (record changes for this file here.)
   CPU accept external interrupt request during ISR.
   v 0.0.0  27 May 2016 by Winson
   Use Interrupt to do something.
*/

#include <stdio.h>
#include <signal.h>
#include "msw.h"
#include "i8048W.h"

void ISRFUN (int slot);
static int rise = 0, fall = 0, ch= 0;
unsigned int TargetCnt=10;

int main()
{
	int Ret, slot, slotIndex=-1, ret_init;
	int rising, falling,  runFlag=0;
  
  Open_Slot(9);        //For interrutp
	for(slot=1;slot<9;slot++)
	{
		Ret = Open_Slot(slot);
		if (Ret == 0) {
		    ret_init=I8048W_Init(slot);
        if (ret_init == 0){        //find I-8048W
              slotIndex=slot;
        }
        else
              Close_Slot(slot);
              // printf("Failed to open module I-8048W on Slot%d.\n",slot);              
		}
	}


  if(slotIndex==-1)
	{
		printf("There is no I-8048W at LinPAC. \n");
		return FAILURE;  // end of program
	}
	else
		printf("There is an I-8048W at slot %d\n", slotIndex);

	printf("Select Channel(0~7):");
	scanf("%d",&ch);  	  
	printf("Set rising enable register(0.Disable 1.Enable):");	
	scanf("%d",&rising);   
	printf("Set falling enable register(0.Disable 1.Enable):");	
	scanf("%d",&falling);  
 	printf("\n");
  
	I8048W_Set_RisingReg(slotIndex, ch, rising);	  //Enable channel 0 as rising edge interrupt.
	I8048W_Set_FallingReg(slotIndex, ch, falling);  //Disable channel 0 as falling edge register.
	I8048W_InstallISR(slotIndex, ISRFUN);
  usleep(500);
	printf("Press 'ESC' and 'Enter' key to go out\n");
  
  runFlag=1;

  while(runFlag==1) 
	{
		if(getch() == 27)  //27 = Esc key 
       runFlag=0;
	}   

  I8048W_UnInstallISR(slotIndex);
	
	printf("Total: channel %d rising edge total count is %d\n",ch, I8048W_Read_RisingEventCount(slotIndex,ch));
	printf("Total: channel %d falling edge total count is %d\n",ch, I8048W_Read_FallingEventCount(slotIndex,ch));

	printf("\nClear rising and falling edge count...\n");
	I8048W_Clear_RisingEventCount(slotIndex,ch);
	I8048W_Clear_FallingEventCount(slotIndex,ch);
	printf("Clearing: slot[%d], channel[%d] rising edge total count is %d\n",slotIndex,ch, I8048W_Read_RisingEventCount(slotIndex,ch));
	printf("Clearing: slot[%d], channel[%d] falling edge total count is %d\n",slotIndex,ch, I8048W_Read_FallingEventCount(slotIndex,ch));
	
  	
  Close_Slot(9); 
	Close_Slot(slotIndex);
	return 0;
}

void ISRFUN (int slot)
{
	int Ret;

	if(I8048W_Read_RisingEvent(slot,ch) != 0)  //rising INT status enable
	{
		printf("ISRFUN: channel %d rising edge trigger %d times.\n",ch, ++rise);
	}

	if(I8048W_Read_FallingEvent(slot,ch) != 0)  //falling INT status enable
	{
		printf("ISRFUN: channel %d falling edge trigger %d times.\n",ch, ++fall);
	}
  
	I8048W_UnFreezeINT(slot,ch);  //Clear interrupt status of I8048W 

}
