/*  Demo program for LinPAC-5000 + XW-304

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   v 0.0.0  1 May 2012 by Cindy Huang
     create, blah blah... */

/*****************************************************************/
/*  XW304:  6 channel 12-Bit A/D (Analog input) +/-5.0V or 0~5V  */
/*          1 channel 12-Bit D/A (Analog output) +/-5.0V         */
/*          4 channel D/I                                        */
/*          4 channel D/O                                        */
/*****************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include "msw.h"
#include "xwboard.h"
#include "xw304.h"


int main()
{
	int quit=0, i,ret,type,wRetVal,dioneVal,diallVal,iDOValue,iChannel,iState;
  float fValue;
  
	/* open device file */
	wRetVal = Open_Slot(SLOT1);
	if (wRetVal > 0) {
		printf("open Slot failed!\n");
		return (-1);
	}
	
	if((ret=XW304_Init())!=0)
  {
        printf("XW304 initiate failed [%d] \n", ret);
        return (-1);
  }
  
	while (quit!=1)
  {
      printf("\n\r");
      printf("         Demo program for LinPAC-5000-XW304          \n\r");
      printf("=====================================================\n\r");
      printf(" Please choose one of the following :                \n\r");
      printf("            0.) Read the calibration data            \n\r");
      printf("            1.) Analog Input(+/-5V)                  \n\r");
      printf("            2.) Analog Input(0~5V))                  \n\r");
      printf("            3.) Analog Output(0~5V)                  \n\r");   
      printf("            4.) Read All DI channels                 \n\r");   
      printf("            5.) Read One DI channel                  \n\r"); 
      printf("            6.) Write All DO channels                \n\r");
      printf("            7.) Write One DO channel                 \n\r"); 
      printf("            8.) DO Readback All channels             \n\r");
      printf("            9.) DO Readback One channel              \n\r");
      printf("            10.) Exit                                \n\r");
      printf(" Note1:AI(Ch0~5);AO(Ch0);DO(Ch0~3);DI(Ch0~3)         \n\r");
      printf("=====================================================\n\r");
      printf("Choose an option and press [Enter]: ");
      scanf("%d",&type);
      printf("\n");
   
	   switch (type)   
	   {
       case 0:
            if((ret&0x01)==0)
                 printf("EEPROM A/D Gain   ==> [%8.6f]\r\n", XW304_Read_AD_Gain(AI_TYPE1)); 
            else
                 printf("No setting in EEPROM\r\n");
               
            if((ret&0x02)==0)
                 printf("EEPROM A/D Offset ==> [%8.6f]\r\n", XW304_Read_AD_Offset(AI_TYPE1)); 
            else
                 printf("No setting in EEPROM\r\n");
                    
            if((ret&0x04)==0)
                 printf("EEPROM A/D Gain   ==> [%8.6f]\r\n", XW304_Read_AD_Gain(AI_TYPE2)); 
            else
                 printf("No setting in EEPROM\r\n");
                
            if((ret&0x08)==0)
                 printf("EEPROM A/D Offset ==> [%8.6f]\r\n", XW304_Read_AD_Offset(AI_TYPE2)); 
            else
                 printf("No setting in EEPROM\r\n");
                
            if((ret&0x10)==0)
                 printf("EEPROM D/A Gain   ==> [%8.6f]\r\n", XW304_Read_DA_Gain());
            else
                 printf("No setting in EEPROM\r\n");

            if((ret&0x20)==0)
                 printf("EEPROM D/A Offset ==> [%8.6f]\r\n", XW304_Read_DA_Offset());
            else
                 printf("No setting in EEPROM\r\n");
            break;
       
       case 1:   // A/D (mA)(Analog input)
            /* Note that the JP2 jumper must be set on pin 1 and 2 (-5 ~ 5V) */
            printf("Channel (0~5): ");
            scanf("%d", &iChannel);
                
            if(iChannel>5)
            {
                printf("Unknow Channel Number!!\r\n");
                break;
            }
            printf("AI%d => %6.4fV\r\n", iChannel, XW304_AnalogIn(AI_TYPE1, iChannel));
            break;
       case 2: 
            /* Note that the JP2 jumper must be set on pin 2 and 3 (0 ~ 5V) */
            printf("Channel (0~5): ");
            scanf("%d", &iChannel);
                
            if(iChannel>5)
            {
                printf("Unknow Channel Number!!\r\n");
                break;
            }
                
            printf("AI%d => %6.4fV\r\n", iChannel, XW304_AnalogIn(AI_TYPE2, iChannel));
            break;
       case 3:  
            printf("Voltage (-5.0 ~ +5.0): ");
            scanf("%f", &fValue);
                
            XW304_AnalogOut(fValue);
            printf("AO => %6.4fV\r\n", fValue);
            
            break;
            
       case 4:    
            printf("Input state => 0x%02X\r\n", XW304_Read_All_DI());
            break;
            
       case 5:   
            printf("DI Channel (0~3): ");
            scanf("%d", &iChannel);
                
            if(iChannel>3)
            {
                printf("Unknow Channel Number!!\r\n");
                break;
            }
                
            iDOValue=XW304_Read_One_DI(iChannel);
            printf("DI%d => %s\r\n", iChannel, iDOValue==0?"ON":"OFF");
				
            break;
       case 6:   
            printf("DO Output state (0~f): ");
            scanf("%x", &iDOValue);
              
            XW304_Write_All_DO(iDOValue);
            printf("Output state => 0x%02X\r\n", iDOValue);
            break;
            
        case 7:           
            printf("DO Channel (0~3): ");
            scanf("%d", &iChannel);   
            if (iChannel>3)
            {
               printf("Unknow Channel Number!!\r\n");
               break;
            }
            
            printf("DO status (0:OFF, 1:ON): ");
            scanf("%d" , &iState);
            if ((iState>1)||(iState<0))
            {
               printf("Unknow state!!\r\n");
               break;
            }    
            XW304_Write_One_DO(iChannel, iState);
            printf("DO%d => %s\r\n", iChannel, iState==0?"OFF":"ON");
       
            break;
            
       case 8: 
            printf("DO ReadBack => 0x%02X\r\n", XW304_Read_All_DO());
            break;
            
       case 9: 
            printf("DO Channel (0~3): ");
            scanf("%d", &iChannel);
                
            if(iChannel>3)
            {
                printf("Unknow Channel Number!!\r\n");
                break;
            }
                
            iDOValue=XW304_Read_One_DO(iChannel);
            printf("DO%d => %s\r\n", iChannel, iDOValue==0?"OFF":"ON");
            break;
            
       case 10: 
            quit=1;
            break;     
      }
  }    // while   
	
	Close_Slot(SLOT1);
	return SUCCESS;
}
